# This Build.PL for Dist-Zilla-PluginBundle-Author-ETHER was generated by Dist::Zilla::Plugin::ModuleBuildTiny 0.014.
use strict;
use warnings;

# inserted by Dist::Zilla::Plugin::CheckBin 0.007
use Devel::CheckBin;
check_bin('git');

use 5.010;
use Module::Build::Tiny 0.034;
Build_PL();
